@@ -14,7 +14,16 @@
 #    under the License.
 
 import collections
+
+# (rpittau) this allows usage of collection ABC abstract classes in both
+# Python 2.7 and Python 3.8+
+try:
+    collectionsAbc = collections.abc
+except AttributeError:
+    collectionsAbc = collections
+
 import logging
+import threading
 
 import six
 
@@ -66,13 +75,17 @@ def int_or_none(x):
     return int(x)
 
 
-def get_sub_resource_path_by(resource, subresource_name):
+def get_sub_resource_path_by(resource, subresource_name, is_collection=False):
     """Helper function to find the subresource path
 
     :param resource: ResourceBase instance on which the name
         gets queried upon.
     :param subresource_name: name of the resource field to
         fetch the '@odata.id' from.
+    :param is_collection: if `True`, expect a list of resources to
+        fetch the '@odata.id' from.
+    :returns: Resource path (if `is_collection` is `False`) or
+        a list of resource paths (if `is_collection` is `True`).
     """
     if not subresource_name:
         raise ValueError('"subresource_name" cannot be empty')
@@ -88,12 +101,23 @@ def get_sub_resource_path_by(resource, subresource_name):
         raise exceptions.MissingAttributeError(
             attribute='/'.join(subresource_name), resource=resource.path)
 
-    if '@odata.id' not in body:
-        raise exceptions.MissingAttributeError(
-            attribute='/'.join(subresource_name) + '/@odata.id',
-            resource=resource.path)
+    elements = []
 
-    return body['@odata.id']
+    try:
+        if is_collection:
+            for element in body:
+                elements.append(element['@odata.id'])
+            return elements
+
+        return body['@odata.id']
+
+    except (TypeError, KeyError):
+        attribute = '/'.join(subresource_name)
+        if is_collection:
+            attribute += '[%s]' % len(elements)
+        attribute += '/@odata.id'
+        raise exceptions.MissingAttributeError(
+            attribute=attribute, resource=resource.path)
 
 
 def max_safe(iterable, default=0):
@@ -215,7 +239,7 @@ def cache_it(res_accessor_method):
 
         if isinstance(cache_attr_val, base.ResourceBase):
             cache_attr_val.refresh(force=False)
-        elif isinstance(cache_attr_val, collections.Sequence):
+        elif isinstance(cache_attr_val, collectionsAbc.Sequence):
             for elem in cache_attr_val:
                 if isinstance(elem, base.ResourceBase):
                     elem.refresh(force=False)
@@ -244,7 +268,7 @@ def cache_clear(res_selfie, force_refresh, only_these=None):
     cache_attr_names = setdefaultattr(
         res_selfie, CACHE_ATTR_NAMES_VAR_NAME, set())
     if only_these is not None:
-        if not isinstance(only_these, collections.Sequence):
+        if not isinstance(only_these, collectionsAbc.Sequence):
             raise TypeError("'only_these' must be a sequence.")
 
         cache_attr_names = cache_attr_names.intersection(
@@ -257,7 +281,7 @@ def cache_clear(res_selfie, force_refresh, only_these=None):
 
         if isinstance(cache_attr_val, base.ResourceBase):
             cache_attr_val.invalidate(force_refresh)
-        elif isinstance(cache_attr_val, collections.Sequence):
+        elif isinstance(cache_attr_val, collectionsAbc.Sequence):
             for elem in cache_attr_val:
                 if isinstance(elem, base.ResourceBase):
                     elem.invalidate(force_refresh)
@@ -266,3 +290,62 @@ def cache_clear(res_selfie, force_refresh, only_these=None):
                     break
         else:
             setattr(res_selfie, cache_attr_name, None)
+
+
+def camelcase_to_underscore_joined(camelcase_str):
+    """Convert camelCase string to underscore_joined string
+
+    :param camelcase_str: The camelCase string
+    :returns: the equivalent underscore_joined string
+    """
+    if not camelcase_str:
+        raise ValueError('"camelcase_str" cannot be empty')
+
+    r = camelcase_str[0].lower()
+    for i, letter in enumerate(camelcase_str[1:], 1):
+        if letter.isupper():
+            try:
+                if (camelcase_str[i - 1].islower()
+                        or camelcase_str[i + 1].islower()):
+                    r += '_'
+            except IndexError:
+                pass
+
+        r += letter.lower()
+
+    return r
+
+
+def synchronized(wrapped):
+    """Simple synchronization decorator.
+
+    Decorating a method like so:
+
+    .. code-block:: python
+
+      @synchronized
+      def foo(self, *args):
+        ...
+
+    ensures that only one thread will execute the foo method at a time.
+    """
+    lock = threading.RLock()
+
+    @six.wraps(wrapped)
+    def wrapper(*args, **kwargs):
+        with lock:
+            return wrapped(*args, **kwargs)
+
+    return wrapper
+
+
+_REMOVE = frozenset(['password', 'x-auth-token'])
+
+
+def sanitize(item):
+    """Remove passwords from the item."""
+    if isinstance(item, dict):
+        return {key: ('***' if key.lower() in _REMOVE else sanitize(value))
+                for key, value in item.items()}
+    else:
+        return item
